;; This file is part of The Angel's Choice.

;; The Angel's Choice is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.

;; The Angel's Choice is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.

;; You should have received a copy of the GNU General Public License along with Foobar. If not, see https://www.gnu.org/licenses/.

.module Entity

;; Start of _CODE area
.area _CODE
.include "man/datos_entity.inc"
.include "sys/globals_render.inc"
.include "sys/globals_generator.inc"
.include "sys/globals_phy.inc"
.include "funciones_cpct.inc"

.globl posiciones_tomadas
entities:: .ds #tam_ent*cant_ent
ent_zero_end :: .db 0
reserved_entities :: .db 0

entities_created :: .db 0

man_entity_init::
   ld a, #0
   ld (ent_zero_end), a
   ld (reserved_entities), a
   ld (entities_created), a
   ;(2B DE) array	Pointer to the first byte of the array to be filled up (starting point in memory)
   ;(1B A ) value	8-bit value to be set
   ;(2B BC) size	Number of bytes to be set (>= 2)
   ld de,#entities
   ld a,#0
   ld bc,#(tam_ent*cant_ent)
   call cpct_memset_asm
ret

man_create_entity::
   ld hl,#entities ;cargo la posicion inicial
   ld a,(reserved_entities)
   ld c,a
   ld b, #0
   add hl,bc ;añade reserved_entities a la posicion inicial
   add #tam_ent ;añado una entidad, dado que creo una nueva: reserved + 5 nueva next
   ld (reserved_entities),a
   ld (hl),#1; e_type_default
   ld a, (entities_created)
   add #1
   ld (entities_created),a
ret

man_entity_for_all::
      ld (pos_a_cambiar+1),hl
      ld ix,#entities ;cargo la posicion inicial
      bucle_para_cada:
         ld a,e_type(ix)
         cp #0
         jr z, fin_mod
            pos_a_cambiar:
            call sys_physics_update_one
            ld b,#0
            ld c,#tam_ent
            add ix,bc
         jp bucle_para_cada
fin_mod:
   ret


man_entity_for_all_matching::
      ld (pos_a_cambiar_match+1),hl
      ld (type_a_cambiar+1),a
      ld ix,#entities ;cargo la posicion inicial
      bucle_para_cada_match:
         ld a,e_type(ix)
         cp #0
         jr z, fin_mod_match
            type_a_cambiar:
            and #e_type_physics
            jr z,no_physics
               pos_a_cambiar_match:
            call sys_physics_update_one
         no_physics:
            
            ld b,#0
            ld c,#tam_ent
            add ix,bc
         jp bucle_para_cada_match
fin_mod_match:
   ret

man_entity_update::
   ld hl, #entities
   recorrer_entities:
      ld a,(hl)
      cp #0
      jr z, fin_muertes
      ld a,(hl)
      and #e_type_dead
      jr z,no_muere
         call man_destroy_entity
         call sys_restar_enemigo
         jp recorrer_entities
   no_muere:
      ld b,#0
      ld c,#tam_ent
      add hl,bc
   jp recorrer_entities

fin_muertes:
ret

man_set_4_destroy::
   ld a, e_type(ix)
   or #e_type_dead ;hace un or con 0x80, debería quedarse como 0x81
   ld e_type(ix),a

   ld hl, #posiciones_tomadas
   ld b, #0
   ld a, e_plat(ix)
   ld c,a
   add hl,bc
   ld (hl), #0

ret

man_destroy_entity::
   ;(2B DE) to	Pointer to the destination (first byte where bytes will be written)
   ;(2B HL) from	Pointer to the source (first byte from which bytes will be read)
   ;(2B BC) size	Number of bytes to be set (>= 1)
   ld d,h ;meto la entidad muerta en de
   ld e,l
   push de
   ld hl,#entities ;meto el comienzo del array
   ld a, (reserved_entities) ;y la cantidad de entidades reservadas (multiplo de 5)
   sub #tam_ent ;le quito 5 para que guarde una entidad menos
   ld c,a
   ld b,#0
   add hl,bc ;añado a la direccion del comienzo la cantidad de posiciones a mover hasta llegar a la ultima
   ld b,#0
   ld c, #tam_ent
   push hl ;guardamos la posicion de la ultima entidad
   ldir ; call cpct_memcpy_asm
   ;en este momento hl apunta al final del array, justo después de la última entidad 
   pop de ;la devolvemos en de para poder eliminar su contenido
   pop hl ;devolvemos en hl la posicion donde se ha guardado la ultima ahora
   

   ;(2B DE) array	Pointer to the first byte of the array to be filled up (starting point in memory)
   ;(1B A ) value	8-bit value to be set
   ;(2B BC) size	Number of bytes to be set (>= 2)
   ld a,#0
   ld bc,#tam_ent
   call cpct_memset_asm
   

   ld a, (reserved_entities) ;y la cantidad de entidades reservadas (multiplo de 5)
   sub #tam_ent ;le quito 5 para que guarde una entidad menos
   ld (reserved_entities),a
    ld a, (entities_created)
   sub #1
   ld (entities_created),a

ret


;;;LE LLEGA EN A UN INDICE Y DEVUELVE EN IX LA POSICION (SI ES 0, FUERA DE RANGO)
man_get_from_idx::
   cp #cant_ent
   jr nc, fuera_de_rango ;;si no hay carry es porque el indice es mayor que la cantidad posible
      cp #0 ;;primera entidad
      jr nz, otra_entidad
         ld ix, #entities
         ret
   otra_entidad:
      ld ix, #entities
      ld b,#0
      ld c, #tam_ent
      loop_buscando_la_entidad:
         cp #0
         jr z, salimos
         add ix, bc
         sub #1
         jr loop_buscando_la_entidad
fuera_de_rango:
   ld ix, #entities
salimos:
ret



man_free_space::
   ld a, (entities_created)
   ld b,a
   ld a, #cant_ent
   sub b
ret