;; This file is part of The Angel's Choice.

;; The Angel's Choice is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.

;; The Angel's Choice is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.

;; You should have received a copy of the GNU General Public License along with Foobar. If not, see https://www.gnu.org/licenses/.

.module Camera_System


.include "man/datos_entity.inc"
.include "sys/globals_render.inc" 
.include "man/globals_ent.inc"
.include "funciones_cpct.inc"

.area _CODE
;; estructura camara
cam_ptr = 0
cam_x = 2
cam_y = 3
cam_x_follow = 4
cam_y_follow = 5
cam_scr_pending = 6


sys_camera_update::
    call control_scroll_up_down
    ld hl, #sys_camera_update_one 
    ld a, #(e_type_render)
    call man_entity_for_all_matching
ret

sys_camera_update_one:
    ;; Calculamos coordenadas de pantalla para una entidad
    ld a, (tilemap + cam_x) ;; coordenada mundo (X)
    ld b, a

    ld a, (tilemap + cam_y) ;; coordenada mundo (Y)
    ld c, a

    ;; coordenada X de la pantalla 
    ld a, e_x(ix)
    sub b
    ld e_x_screen(ix), a

    ;; coordenada Y de la pantalla (es la que moveremos con el scroll)
    ld a, e_y(ix)           ;; Coordenada Y (Bytes World)
    sub c                   ;; y -= cam_y   (Bytes)
    ld e_y_screen(ix), a    ;; e_y_screen = y - cam_y

    ;; Actualizar coordenadas de seguimiento
    ld a, e_type(ix)
    and #e_type_input
    jr z,nada_mas
        ld a, e_y_screen(ix)
        ld (tilemap + cam_y_follow), a
    nada_mas:
    
ret

control_scroll_up_down:
    ;; comprobar limites del mapa 
    ld ix, #entities
    ld a, e_y(ix)
    cp #50
    ret c
    ld a, e_y(ix)
    cp #160
    ret nc

    ;; hacer scroll cuando este en el estado scroll
    ld a, (tilemap + cam_scr_pending)
    bit 7, a
    jr nz, scroll_up
    or a
    jr nz, scroll_down

    ;; detectar inicio del scroll
    ld a, (tilemap + cam_y_follow)
    cp #80
    jp nc, start_scroll_down
    cp #23
    jp c, start_scroll_up

ret



scroll_up:
    ld hl, #(tilemap + cam_scr_pending)
    inc (hl)

;;    *** SCROLL UP ***
    ;; mover camara ptr
    ld hl, (tilemap) ;; tilemap camera ptr
    ld de, #-20                 ;; Ancho del mapa (se salta una fila)
    add hl, de
    ld (tilemap), hl

    ;; actualizar coordenadas de la camara
    ld a, (tilemap + cam_y)
    add #-8                     ;; alto/2 del sprite 
    ld (tilemap + cam_y), a
ret

scroll_down:
    ld hl, #(tilemap + cam_scr_pending)
    dec (hl)

;;    *** SCROLL DOWN *** 
    ;; mover camara ptr
    ld hl, (tilemap) ;; tilemap camera ptr
    ld de, #20
    add hl, de
    ld (tilemap), hl

    ;; actualizar coordenadas de la camara
    ld a, (tilemap + cam_y)
    add #8
    ld (tilemap + cam_y), a
ret


start_scroll_down:
    ld a, #6
    ld (tilemap + cam_scr_pending), a
ret

start_scroll_up:
    ld a, #-6
    ld (tilemap + cam_scr_pending), a
ret