;; This file is part of The Angel's Choice.

;; The Angel's Choice is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.

;; The Angel's Choice is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.

;; You should have received a copy of the GNU General Public License along with Foobar. If not, see https://www.gnu.org/licenses/.

.module Render

.include "cpctelera.h.s"
;; Start of _CODE area
.area _CODE
.include "man/datos_entity.inc"
.include "man/globals_ent.inc"
.include "menus/datos_menu.inc"
.include "sys/globals_hud.inc"
.include "sys/globals_camera.inc" ;; scroll
.include "man/globals_maps.inc" 
.include "funciones_cpct.inc"
.include "assets/assets.inc"


.globl _g_palette
.globl _tile_00

hud_render:
  .dw 0
hud_render_1:
  .dw 0

_back_buffer:: .dw 0x8000
tilemap:: .dw 0x00f0;0x04B0 ;; Tilemap camera pointer >>>> (alto - ancho) x ancho >>>> mapa 32x20 -> 12x20 = 240 = 0x00f0
          .db 0, #0x60   ;; Coordenadas x, y de la camara en el mundo
          .db 0, #0x8c  ;; Coordenadas x, y de seguimiento en la pantalla
          .db #0     ;; Tiles de los que tengo que hacer scroll

tilemap_size = . - tilemap

sys_render_init::
    ;; modo video 
   ld c, #0
   call cpct_setVideoMode_asm

   ;; borde 
   cpctm_setBorder_asm HW_BLACK

   ;; array de colores
   ld hl, #_g_palette
   ;; num colores
   ld de, #16
   call cpct_setPalette_asm

  
ret

sys_render_init_level::
   ;; init camera
   call sys_camera_init
ret

sys_camera_init::
  ;  ;; init camera
   ld hl, #tilemap
   ld (hl), #0xf0
   inc hl ;segundo de tilemap
   ld (hl), #0x00
   inc hl ;coordenada x
   ld (hl), #0x00
   inc hl ;coordenada y
   ld (hl), #0x60
   inc hl ;coordenada x
   ld (hl), #0x00
   inc hl ;coordenada y
   ld (hl), #0x8c
   
ret

;; rectangulo del hud
sys_render_UI_2:: 
   ;call sys_render_tilemap ;; solo si queremos dibujar 1 unica vez
   ld hl,#0x8000
   call sys_render_UI
   ld hl,#0xC000
   call sys_render_UI
ret


;; -----------------------------------------------------------------
;; Pintar el mapa
;; -----------------------------------------------------------------
sys_render_tilemap:

    ;; **  AJUSTAMOS EL TILEMAP PARA PINTARLO  **
    ld    bc, #0x1414                       ;; ancho y alto en tiles de lo que se va a dibujar(hexadecimal)
    ld    de, #0x14                         ;; total del ancho (hexadecimal)
    ld    hl, #_tile_00                     ;; puntero al tileset
    call cpct_etm_setDrawTilemap4x8_ag_asm


    ; ;; **      DIBUJAMOS EL TILEMAP COOO   **
    ; ld hl, #_tilemap_01          ;; Cargo el puntero inicial al tilemap
    ; ld de, (tilemap)             ;; Para bajar una fila de tiles multiplico la mitad del tilemap (16) x ancho (32) = 512 (0x200)         
    ; add hl, de                   ;; Se lo sumo al puntero
    ; ex de, hl
    ; ld hl, #0xC000
    ; call cpct_etm_drawTilemap4x8_ag_asm

    ;; **      DIBUJAMOS EL TILEMAP  8000  ***
    ;ld hl, #_tilemap_01          ;; Cargo el puntero inicial al tilemap
    call man_get_current_map
    ex de,hl
    ld de, (tilemap)             ;; Para bajar una fila de tiles multiplico la mitad del tilemap (16) x ancho (32) = 512 (0x200)         
    add hl, de                   ;; Se lo sumo al puntero
    ex de, hl
    ld hl, (#_back_buffer) 
    call cpct_etm_drawTilemap4x8_ag_asm

ret

sys_render_update::
    call sys_render_tilemap
    ld a, #e_type_render
    ld hl,#sys_render_update_one 
    call man_entity_for_all_matching

    call render_hud
    call change_screen
ret

sys_render_update_one::
  call render_entity
ret

render_entity:

  ;; si se sale del alto de la pantalla no se pinta
  ld a, e_y_screen(ix)                  ;; B = y coordinate (24 = 0x18)
  ld b, a
  cp #145                ;; Amstrad modo 0 (alto x ancho) -> 160x200 px = 51x64 Bytes // jugador 10x16 px = 3x5 Bytes >>>>> 51-3 = 48 (alto)
  ret nc
  cp #0
  ret c

  ; Calculate a video-memory location for printing a string
  ld de, (#_back_buffer)          ;; DE = Pointer to start of the screen
  
  ; ld c, e_x(ix)                  ;; C = x coordinate (16 = 0x10)
  ; ld b, e_y(ix)                  ;; B = y coordinate (24 = 0x18)
  ld c, e_x_screen(ix)                  ;; C = x coordinate (16 = 0x10)

  call cpct_getScreenPtr_asm     ;; Calculate video memory location and return it in HL

  ex de, hl
  ; (2B HL) sprite    Source Sprite Pointer (array with pixel and mask data)
  ; (2B DE) memory    Destination video memory pointer
  ; (1B C ) width    Sprite Width in bytes (>0) (Beware, not in pixels!)
  ; (1B B ) height    Sprite Height in bytes (>0)
  ld h, e_sprite_h(ix)
  ld l, e_sprite_l(ix)

  ld b, e_height(ix)
  ld c, e_width(ix)
  call cpct_drawSpriteMasked_asm
ret



;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;  Dibuja el hud siempre que se cambie de mapa
;; Se introduce en hl el buffer donde dibujar ( 8000 o C000 )
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
sys_render_UI::

    push hl
    ld a, (cambio + 1)
    cp #1
    jr z, estamos_en_el_cielo
      ld hl, #_hud_infierno_0
      ld (hud_render), hl
      ld hl, #_hud_infierno_1
      ld (hud_render_1), hl
      jr continua
    estamos_en_el_cielo:
      ld hl, #_hud_cielo_0
      ld (hud_render), hl
      ld hl, #_hud_cielo_1
      ld (hud_render_1), hl
  continua:
    pop hl 
    push hl
      ex de, hl
      ld bc, #0xA000
      call cpct_getScreenPtr_asm
      ex de, hl
      ld c, #40
      ld b, #20
      ld hl, (hud_render)
      call cpct_drawSprite_asm
      pop hl
      push hl
      ld de, (hud_render)
      ld bc, #0xC700
      call render_flip_h

      pop hl
      push hl
      ex de, hl
      ld bc, #0xA028
      call cpct_getScreenPtr_asm
      ex de, hl
      ld c, #40
      ld b, #20
      ld hl, (hud_render_1)
      call cpct_drawSprite_asm
      pop hl
      ld de, (hud_render_1)
      ld bc, #0xC728
      call render_flip_h
;   ;; ================================================== ;
   ret

render_flip_h:
  ;(2B DE) sprite	Source Sprite Pointer
  ;(2B HL) memory	Destination video memory pointer (Bottom-left corner)
  ;(1B C ) width	Sprite Width in bytes (Beware, not in pixels!)
  ;(1B B ) height	Sprite Height in bytes
  push de
     ;; Calculate a video-memory location for printing a string
   ex de, hl ;; DE = Pointer to start of the screen
   ;ld    b, #0                  ;; B = y coordinate (24 = 0x18)
   ;ld    c, #40               ;; C = x coordinate (16 = 0x10)
   call cpct_getScreenPtr_asm    ;; Calculate video memory location and return it in HL
   ld c,#40
   ld b,#20
   pop de
   ;ld hl, #_fondo
   call cpct_drawSpriteVFlip_f_asm
ret


; ;; ============================================================ ;
;; **
;;                                          DOBLE BUFFER
;; **

;; -----------------------------------------------------------------
;;  Cambia pantalla automaticamente alternando entre 8000 y C000 
;; -----------------------------------------------------------------
change_screen:
  f_change_screen = . + 1
  jp change_screen_to_8000

;; -----------------------------------------------------------------
;;  Cambia pantalla a 8000
;; -----------------------------------------------------------------
change_screen_to_8000:
  ld bc, #0x200C
  call cpct_setCRTCReg_asm

  ld hl, #0xC000
  ld (_back_buffer), hl

  ld hl, #change_screen_to_C000
  ld (f_change_screen), hl

  ret

;; -----------------------------------------------------------------
;;  Cambia pantalla a C000
;; -----------------------------------------------------------------
change_screen_to_C000::
  ld bc, #0x300C
  call cpct_setCRTCReg_asm

  ld hl, #0x8000
  ld (_back_buffer), hl

  ld hl, #change_screen_to_8000
  ld (f_change_screen), hl

  ret


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;FUNCIONES DEL MENÚ


;;;recibe en b el add que se le hará a la frase y en h el color de fondo
dibujar_frase::
  push bc
    ;; Set up draw char colours before calling draw string
   ;ld    h, #0         ;; D = Background PEN (0)
   ld    l, m_color(ix)         ;; E = Foreground PEN (3)

   call cpct_setDrawCharM0_asm   ;; Set draw char colours

   ;; Calculate a video-memory location for printing a string
   ld   de, #CPCT_VMEM_START_ASM ;; DE = Pointer to start of the screen
   ld   a, m_x(ix)   
   pop bc
   add b 
   ld    c,a            ;; B = y coordinate (24 = 0x18)
   ld    b, m_y(ix)            ;; C = x coordinate (16 = 0x10)

   call cpct_getScreenPtr_asm    ;; Calculate video memory location and return it in HL

   ;; Print the string in video memory
   ;; HL already points to video memory, as it is the return
   ;; value from cpct_getScreenPtr_asm
   ;ld   iy, #string    ;; IY = Pointer to the string 

   call cpct_drawStringM0_asm  ;; Draw the string
ret

dibujar_sprite::
 ;; Calculate a video-memory location for printing a string
   ld   de, #CPCT_VMEM_START_ASM ;; DE = Pointer to start of the screen
   ld    b, m_y(ix)                  ;; B = y coordinate (24 = 0x18)
   ld    c, m_x(ix)                  ;; C = x coordinate (16 = 0x10)

   call cpct_getScreenPtr_asm    ;; Calculate video memory location and return it in HL

    ex de,hl
   ;(2B HL) sprite	Source Sprite Pointer (array with pixel data)
   ;(2B DE) memory	Destination video memory pointer
   ;(1B C ) width	Sprite Width in bytes [1-63] (Beware, not in pixels!)
   ;(1B B ) height	Sprite Height in bytes (>0)
   
    ld h, m_sprite_h(ix)
    ld l, m_sprite_l(ix)
   ld c,m_width(ix)
   ld b,m_height(ix)
   call cpct_drawSprite_asm
ret

;;recibe en b el alto y en c el ancho y en hl el sprite
render_fondo_menu::
  ld b, #0
  ld c, #0
  call render_part
  ld b, #0
  ld c, #40
  call render_part
  ld b, #100
  ld c, #40
  call render_part
  ld b, #100
  ld c, #0
  call render_part

ret

render_part:
     ;; Calculate a video-memory location for printing a string
   ld   de, #CPCT_VMEM_START_ASM ;; DE = Pointer to start of the screen
   ;ld    b, #0                  ;; B = y coordinate (24 = 0x18)
   ;ld    c, #40               ;; C = x coordinate (16 = 0x10)
   call cpct_getScreenPtr_asm    ;; Calculate video memory location and return it in HL
   ex de, hl
   ld c,#40
   ld b,#100
   ld a, #0xCF

  ;(2B DE) memory	Video memory pointer to the upper left box corner byte
  ;(1B A ) colour_pattern	1-byte colour pattern (in screen pixel format) to fill the box with
  ;(1B C ) width	Box width in bytes [1-64] (Beware!  not in pixels!)
  ;(1B B ) height	Box height in bytes (>0)

    call cpct_drawSolidBox_asm
ret

render_completo::
;; descomprimir fondo
    ld hl, #_screen_menu_z_end
    ld de, #0xFFFF
    ;; (2B DE) dest_end    Ending (latest) byte of the destination (decompressed) array
    ;; (2B HL) source_end    Ending (latest) byte of the source (compressed) array
    call cpct_zx7b_decrunch_s_asm

ret

